/*
 *  DeafGuard Alert System
 * 
 *  What it does: 
 *   This sketch integrates hardware sensors for acquiring and accessing information in a monitoring system, 
 *   designed to assist deaf individuals during emergency situations such as alarms. 
 *   The system includes a moisture sensor as an SOS button, 
 *   an accelerometer as an activity indicator, and a servo motor to notify, 
 *   with real-time updates displayed on the Blynk dashboard.
 * 
 *  The circuit:
 *  Input:
 *  - Moisture sensor
 *  - Accelerometer (MPU6050)
 *  - remote Blynk Dashboard switch for viberation
 *  - pkr-API alert for alarms(TZEVA ADOM).
 *  
 *  Output:
 *  - Servo motor for vibrating button
 *  - Real-time updates on the Blynk dashboard
 *  
 *  Created by:
 *  - Itai_Symon 
 *  - Roy_Yotam 
 */
 
/* Fill-in information from Blynk Device Info here */
#define BLYNK_TEMPLATE_ID // put your template id here
#define BLYNK_TEMPLATE_NAME // put the template name here
#define BLYNK_AUTH_TOKEN // put your auth token here

/* Comment this out to disable prints and save space */
#define BLYNK_PRINT Serial

// Blynk
#include <WiFi.h>
#include <WiFiClient.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include <BlynkSimpleEsp32.h>

// WiFi credentials.
char ssid[] = ; // put your wifi ssid here
char pass[] = ; // put your wifi password here

#include "I2Cdev.h"
#include "MPU6050.h"
#include <ESP32Servo.h>

#define BUTTON_PIN 32 //moisture sensor
#define SERVO_PIN 15

//moisture btn value
int btn_value = 1;

Servo myservo;
int servoPos = 90;    // variable to store the servo position
int servoInc = 1;
bool servoOn = false;

// only using ax
MPU6050 accelgyro;
int16_t ax, ay, az;
int16_t gx, gy, gz;
//ax  value
int ax_value = 0;
int isActive = 0;

unsigned long startingViberationTime = 0;
unsigned long timeDifference = 0;

unsigned long APIRequestTimer = 0;
const char* myLocation = "תל אביב";


// Global variable with the hardcoded JSON response
const char* hardcodedApiResponse = "["
  "{\"notificationId\":\"315de9a7-fdb5-403a-be97-58e8a18b5e56\","
  "\"time\":1698668959,"
  "\"threat\":0,"
  "\"isDrill\":false,"
  "\"cities\":[\"בית מאיר\",\"נווה אילן\",\"שורש\",\"ירושלים - מערב\",\"אבן ספיר\",\"תל אביב\"]},"
  "{\"notificationId\":\"6d6bd4d7-f64f-43d1-a443-cb39c36e6dd1\","
  "\"time\":1698668970,"
  "\"threat\":0,"
  "\"isDrill\":false,"
  "\"cities\":[\"אפרת\",\"נווה דניאל\",\"ירושלים - דרום\",\"אורה\",\"עמינדב\"]}"
"]";

//dashboard owner's viberate.
BLYNK_WRITE(V0)
{
  int pinValue = param.asInt(); // assigning incoming value from pin V0 to a variable

  // process received value
  if(pinValue == 1)
  {
    startingViberationTime = millis();
  }
}

void setup() {
  Serial.begin(115200);

  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, pass, "sgp1.blynk.cloud");
  
  // moisture btn  
  pinMode(BUTTON_PIN, INPUT);

  // servo
  myservo.setPeriodHertz(50);    // standard 50 hz servo
  myservo.attach(SERVO_PIN, 500, 2500); // attaches the servo onto pin

  // accelometer
  Wire.begin();
  Serial.println("Initializing I2C devices...");
  accelgyro.initialize();
  Serial.println("Testing device connections...");
  Serial.println(accelgyro.testConnection() ? "MPU6050 connection successful" : "MPU6050 connection failed");

}

void loop() {
  Blynk.run();
  
  int sos_button = digitalRead(BUTTON_PIN);

  if(btn_value != sos_button)
  {
    Blynk.virtualWrite(1, (1 - sos_button));
    btn_value = sos_button;
  }
  
  accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);
  Serial.printf("moisture btn:%d\n", sos_button); //TODO delete when finished creating
  Serial.printf("servo pos:%d\n", servoPos); //TODO delete when finished creating
  Serial.printf("acc:%d\n", ax); //TODO delete when finished creating
  isActive = (ax > 0 ? 1 : 0);
  
  if(ax_value != isActive)
  {
    Blynk.virtualWrite(2, isActive);
    ax_value = isActive;
  }

   if (((millis() - APIRequestTimer) > 31000) && (receivedAPIVibrate(myLocation)))
  {
    startingViberationTime = millis();
    APIRequestTimer = millis();
  }

  timeDifference = millis() - startingViberationTime ;
  if(timeDifference < 30000 && (startingViberationTime != 0))
  {
    // turn off viberate switch on dashboard when time's up    
    if(timeDifference > 29000)
    {
      /* just for testing the code */
       hardcodedApiResponse = "["
        "{\"notificationId\":\"315de9a7-fdb5-403a-be97-58e8a18b5e56\","
        "\"time\":1698668959,"
        "\"threat\":0,"
        "\"isDrill\":false,"
        "\"cities\":[\"בית מאיר\",\"נווה אילן\",\"שורש\",\"ירושלים - מערב\",\"אבן ספיר\"]},"
        "{\"notificationId\":\"6d6bd4d7-f64f-43d1-a443-cb39c36e6dd1\","
        "\"time\":1698668970,"
        "\"threat\":0,"
        "\"isDrill\":false,"
        "\"cities\":[\"אפרת\",\"נווה דניאל\",\"ירושלים - דרום\",\"אורה\",\"עמינדב\"]}"
      "]";
      /* until here */
      Blynk.virtualWrite(0, 0);
      Blynk.virtualWrite(3, 0);
    }
    
    viberate();
  }
}

void viberate() {
    // goes from 0 degrees to 180 degrees
    // in steps of 1 degree
    myservo.write(servoPos);
    delay(9);             // waits 15ms for the servo to reach the position
    servoPos += servoInc;
    
    if (servoPos == 180 || servoPos == 0)
    {
        servoInc *= -1;
    }
}

bool receivedAPIVibrate(const char* myLocation) {
  String url = "https://api.tzevaadom.co.il/notifications";
  String jsonBuffer = getJsonContent(url.c_str());

  const size_t capacity = JSON_OBJECT_SIZE(5) + 1000;  // Adjusted to JSON_OBJECT_SIZE(5)
  DynamicJsonDocument doc(capacity);

  DeserializationError error = deserializeJson(doc, hardcodedApiResponse);
  if (error) {
    Serial.print("Failed to parse JSON: ");
    Serial.println(error.c_str());
    return false;
  }

  // Assuming there is an array of notifications in the JSON response
  JsonArray notifications = doc.as<JsonArray>();

  // Iterate through each notification
  for (JsonObject notification : notifications) {
    // Extract the cities array from the notification
    JsonArray cities = notification["cities"];

    // Iterate through each city in the cities array
    for (JsonVariant city : cities) {
      // Compare the current city with myLocation
      const char* currentCity = city.as<const char*>();
      if (strcmp(currentCity, myLocation) == 0) {
        // City found, return true, and update dashboard's alert
        Blynk.virtualWrite(3, 1);
        Blynk.virtualWrite(0, 1);
        return true;
      }
    }
  }

  // City not found in any notification, return false
  return false;
}

String getJsonContent(const char* url) {
  String payload = "{}";
  WiFiClientSecure client;
  client.setInsecure();
  HTTPClient https;
  https.begin(client, url);
  int rc = https.GET();
  if (rc > 0) {
    Serial.printf("HTTP Response code: %d\n", rc);
    payload = https.getString();
  }
  else {
    Serial.printf("Error code: %d\n", rc);
  }
  https.end();
  return payload;
}
